@@ -8,6 +8,7 @@ import string
 import time
 import ddtcv
 
+from typing import Union
 from threading import Thread
 from ctypes import windll
 from ddtank import stop_thread
@@ -81,6 +82,7 @@ def get_keycode(key: str) -> int:
     :param key: 按键的名称
     :return: 按键的键值，如果没有找到该键键值，返回-1
     """
+    key = key.lower()
     if len(key) == 1 and key in string.printable:
         return VkKeyScanA(ord(key)) & 0xff
     elif key in keycode_dict.keys():
@@ -92,6 +94,7 @@ def get_keycode(key: str) -> int:
 class Status:
     def __init__(self, hwnd: int, title: str = None):
         self.handle = hwnd
+        self.dc = win32gui.GetWindowDC(self.handle)
         if title:
             self.name = title
         else:
@@ -99,7 +102,7 @@ class Status:
 
         self.thread = None
 
-        self.capture = None
+        self.capture_result = None
         self.image_path = './image'
 
         self.wind = None
@@ -149,9 +152,9 @@ class Status:
             self.__delay(period)
         win32api.SendMessage(self.handle, win32con.WM_KEYUP, keycode, 0)
 
-    def __capture(self, position: tuple = (0, 0, 1000, 600), r: bool = False):
+    def capture(self, position: tuple = (0, 0, 1000, 600), r: bool = False):
         """
-        对角色游戏窗口执行截图操作，默认将截图存储在self.capture而不是返回一个图片，如果需要返回值，请设置r = True
+        对角色游戏窗口执行截图操作，默认将截图存储在self.capture_result而不是返回一个图片，如果需要返回值，请设置r = True
         注意，对于某些浏览器或登陆器，此方法可能无法正确获取截图(表现为全黑色截图)
         因此，您需要自己来选择合适的登陆器以正确运行脚本
         :param position: 指明截图位置的元组，分别为左上角x坐标、左上角y坐标、宽度、高度，默认为(0, 0, 1000, 600)即整个窗口
@@ -174,24 +177,53 @@ class Status:
         img = cv2.cvtColor(img, cv2.COLOR_RGBA2RGB)
         if r:
             return img
-        self.capture = img
+        self.capture_result = img
 
     def activate(self, period: int = 0):
         win32api.PostMessage(self.handle, win32con.WM_SETFOCUS, 0, 0)
         self.__sleep(period)
 
-    def find(self, method: str = 'point', **kwargs):
-        if method in ('point', 'p'):
+    def sleep(self, period: int):
+        self.__sleep(period)
+
+    def find(self, method: str = 'find', **kwargs) -> Union[tuple, str, None]:
+        self.activate()
+        if method in ('find', 'f'):
             pos = kwargs['pos']
-            self.__capture()
-            return self.capture[pos[1], pos[0]][::-1]
-        elif method in ('image', 'i'):
-            part = kwargs['part']
+            pixel = win32gui.GetPixel(self.dc, *pos)
+            red = pixel & 0xff
+            green = (pixel >> 8) & 0xff
+            blue = (pixel >> 16) & 0xff
+            return red, green, blue
+        elif method in ('rfind', 'rf', 'r'):
+            pos = kwargs['pos']
+            while True:
+                self.activate()
+                if self.find('find', pos=pos) == kwargs['rgb']:
+                    break
+                else:
+                    if 'el' in kwargs.keys():
+                        kwargs['el']()
+                    if 'period' in kwargs.keys():
+                        self.__sleep(kwargs['period'])
+                    else:
+                        self.__sleep(100)
+        elif method in ('mfind', 'mf', 'm'):
+            for condition in kwargs.keys():
+                pos: tuple = kwargs[condition][0]
+                rgb: tuple = kwargs[condition][1]
+                if self.find('find', pos=pos) == rgb:
+                    return condition
+        elif method in ('find_image', 'fi', 'image', 'i'):
+            if 'part' in kwargs.keys():
+                part = kwargs['part']
+            else:
+                part = (0, 0, 1000, 600)
             img = kwargs['img']
-            self.__capture()
+            self.capture()
             template_image = cv2.imread(self.image_path + '/' + img + '.png', 0)
             x, y, w, h = part
-            image = cv2.cvtColor(self.capture, cv2.COLOR_BGR2GRAY)
+            image = cv2.cvtColor(self.capture_result, cv2.COLOR_BGR2GRAY)
             res = cv2.matchTemplate(image[y:y+h, x:x+w], template_image, cv2.TM_CCOEFF_NORMED)
             min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(res)
             if max_val >= 0.9:
@@ -201,26 +233,17 @@ class Status:
                 return None
 
     def click(self, method: str = 'normal', **kwargs):
-        def convert(hex_str: str):
-            rgb_list = []
-            for i in range(3):
-                rgb_list.append(int(hex_str[i*2: i*2+2], 16))
-            return tuple(rgb_list)
-        if method in ('normal', 'n'):
+        if method in ('normal', 'n', 'click'):
             pos = kwargs['pos']
             self.__click(pos=pos)
         elif method in ('find', 'f'):
             pos = kwargs['pos']
-            if type(kwargs['rgb']) == str:
-                kwargs['rgb'] = convert(kwargs['rgb'])
-            if (self.find('point', pos=pos) == kwargs['rgb']).all():
+            if self.find('find', pos=pos) == kwargs['rgb']:
                 self.__click(pos=pos)
-        elif method in ('rfind', 'rf'):
+        elif method in ('rfind', 'rf', 'r'):
             pos = kwargs['pos']
-            if type(kwargs['rgb']) == str:
-                kwargs['rgb'] = convert(kwargs['rgb'])
             while True:
-                if (self.find('point', pos=pos) == kwargs['rgb']).all():
+                if self.find('find', pos=pos) == kwargs['rgb']:
                     self.__click(pos=pos)
                     break
                 else:
@@ -229,8 +252,8 @@ class Status:
                     if 'period' in kwargs.keys():
                         self.__sleep(kwargs['period'])
                     else:
-                        self.__sleep(1000)
-        elif method in ('find_image', 'fi'):
+                        self.__sleep(100)
+        elif method in ('find_image', 'fi', 'i'):
             img = kwargs['img']
             if 'part' in kwargs.keys():
                 part = kwargs['part']
@@ -239,7 +262,7 @@ class Status:
             pos = self.find('image', img=img, part=part)
             if pos:
                 self.__click(pos)
-        elif method in ('rfind_image', 'rfi'):
+        elif method in ('rfind_image', 'rfi', 'ri'):
             img = kwargs['img']
             if 'part' in kwargs.keys():
                 part = kwargs['part']
@@ -256,7 +279,7 @@ class Status:
                     if 'period' in kwargs.keys():
                         self.__sleep(kwargs['period'])
                     else:
-                        self.__sleep(1000)
+                        self.__sleep(100)
 
     def press(self, key_series):
         for key in key_series:
@@ -268,23 +291,35 @@ class Status:
                 self.__press(*key)
 
     def read(self, is_circle: bool = False):
+        def cal_time(func):
+            def wrapper(*args, **kwargs):
+                start = time.perf_counter()
+                result = func(*args, **kwargs)
+                print(f'{func.__name__} running time: {time.perf_counter() - start}')
+                return result
+            return wrapper
+
+        @cal_time
         def read_wind():
-            image = self.capture
+            image = self.capture_result
             b, g, r = image.item(21, 468, 0), image.item(21, 468, 1), image.item(21, 468, 2)
             if b == 252 and g == r and g > 240 and r > 240:
                 face = 1
             else:
                 face = -1
-            return ddtcv.Wind(self.capture) * face
+            return ddtcv.Wind(self.capture_result) * face
 
+        @cal_time
         def read_angle():
-            return ddtcv.Angle(self.capture)
+            return ddtcv.Angle(self.capture_result)
 
+        @cal_time
         def read_small_map():
-            return np.argwhere(np.all(self.capture[1, 750:] == [160, 160, 160], axis=-1))[0, 0] + 742
+            return np.argwhere(np.all(self.capture_result[1, 750:] == [160, 160, 160], axis=-1))[0, 0] + 742
 
+        @cal_time
         def read_white_box():
-            img = np.where(np.any(self.capture[24:120, self.map_pos:998] != [153, 153, 153], axis=-1), 0, 255).astype('uint8')
+            img = np.where(np.any(self.capture_result[24:120, self.map_pos:998] != [153, 153, 153], axis=-1), 0, 255).astype('uint8')
             contours, hierarchy = cv2.findContours(img, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
             for cnt in contours:
                 x, y, w, h = cv2.boundingRect(cnt)
@@ -294,49 +329,47 @@ class Status:
                     return white_box_pos, white_box_width
             return None
 
+        @cal_time
         def read_blue():
             blue_list = []
             blue_triangle = None
-            img = np.where(np.any(self.capture[24:120, self.map_pos:998] != [204, 51, 0], axis=-1), 0, 255).astype('uint8')
+            img = np.where(np.any(self.capture_result[24:120, self.map_pos:998] != [204, 51, 0], axis=-1), 0, 255).astype('uint8')
             contours, hierarchy = cv2.findContours(img, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
             for cnt in contours:
                 x, y, w, h = cv2.boundingRect(cnt)
                 if h > 3:
-                    blue = (x, y)
-                    # blue = (x, y, w)
+                    blue = (int(x + w / 2), int(y + h / 2))
                     blue_list.append(blue)
-                elif h == 3 and w == 5:
-                    blue_triangle = (x, y)
+                elif 2 <= h <= 4 and 4 <= w <= 6:
+                    blue_triangle = (int(x + w / 2), int(y + h / 2))
             return blue_list, blue_triangle
 
+        @cal_time
         def read_red():
             red_list = []
-            img = np.where(np.any(self.capture[24:120, self.map_pos:998] != [0, 0, 255], axis=-1), 0, 255).astype('uint8')
+            img = np.where(np.any(self.capture_result[24:120, self.map_pos:998] != [0, 0, 255], axis=-1), 0, 255).astype('uint8')
             contours, hierarchy = cv2.findContours(img, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
             for cnt in contours:
                 x, y, w, h = cv2.boundingRect(cnt)
                 if h > 3:
-                    red = (x, y)
-                    # red = (x, y, w)
+                    red = (int(x + w / 2), int(y + h / 2))
                     red_list.append(red)
             return red_list
 
+        @cal_time
         def read_circle():
             def get_circle():
-                capture_1 = self.__capture(position=(self.map_pos, 24, 998 - self.map_pos, 120 - 24), r=True)
+                self.activate()
+                capture_1 = self.capture(position=(self.map_pos, 24, 998 - self.map_pos, 120 - 24), r=True)
                 capture_1 = cv2.cvtColor(capture_1, cv2.COLOR_BGR2GRAY)
                 self.__sleep(30)
-                capture_2 = self.__capture(position=(self.map_pos, 24, 998 - self.map_pos, 120 - 24), r=True)
+                capture_2 = self.capture(position=(self.map_pos, 24, 998 - self.map_pos, 120 - 24), r=True)
                 capture_2 = cv2.cvtColor(capture_2, cv2.COLOR_BGR2GRAY)
                 image = np.where(capture_1 == capture_2, 255, 0).astype('uint8')
-                contours, hierarchy = cv2.findContours(image, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
-                if len(contours) < 2:
-                    return None
-                if len(contours) > 4:
-                    return None
+                contours, _ = cv2.findContours(image, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
                 for cnt in contours:
                     x, y, w, h = cv2.boundingRect(cnt)
-                    if 10 < w < 50 or 10 < h < 50:
+                    if 15 < w < 25 or 15 < h < 25 and w == h:
                         self_x = int(x + w / 2)
                         self_y = int(y + h / 2)
                         return self_x, self_y
@@ -347,7 +380,7 @@ class Status:
                 else:
                     self.__sleep(100)
 
-        self.__capture()
+        self.capture()
         self.wind = read_wind()
         self.angle = read_angle()
         self.map_pos = read_small_map()
@@ -398,9 +431,9 @@ class Status:
 
             win32api.PostMessage(self.handle, win32con.WM_KEYDOWN, 32, 0)
             self.__delay(100)
-            pixel = self.capture()[y_pos, x_pos]
+            pixel: tuple = self.capture_result[y_pos, x_pos]
             while True:
-                if (self.capture()[y_pos, x_pos] != pixel).any():
+                if (self.capture_result[y_pos, x_pos] != pixel).any():
                     win32api.PostMessage(self.handle, win32con.WM_KEYUP, 32, 0)
                     break
 
@@ -439,3 +472,5 @@ class Status:
 
 
 
+
+
